@@ -1,4 +1,19 @@
-#include "fastcpd_classes.h"
+#include "fastcpd_class.h"
+
+using ::arma::as_scalar;
+using ::arma::eye;
+using ::arma::log_det_sympd;
+using ::arma::ones;
+using ::arma::reverse;
+using ::arma::trace;
+using ::arma::zeros;
+using ::Rcpp::as;
+using ::Rcpp::Named;
+using ::std::pow;
+
+using ::Rcpp::Environment;
+using ::Rcpp::NumericVector;
+using ::Rcpp::S4;
 
 namespace fastcpd::classes {
 
@@ -47,8 +62,7 @@ colvec Fastcpd::get_gradient_arma(
     psi_coefficient.row(segment_length - 1).t() *
     variance_term(segment_length - 1) / theta(sum(order));
   gradient(sum(order)) = 1.0 / 2.0 / theta(sum(order)) -
-    std::pow(variance_term(segment_length - 1), 2) / 2.0 /
-    std::pow(theta(sum(order)), 2);
+    pow(variance_term(segment_length - 1), 2) / 2.0 / pow(theta(sum(order)), 2);
   return gradient;
 }
 
@@ -65,6 +79,16 @@ colvec Fastcpd::get_gradient_binomial(
   return - (y - 1 / (1 + exp(-as_scalar(x * theta)))) * x.t();
 }
 
+colvec Fastcpd::get_gradient_custom(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  const colvec& theta
+) {
+  return as<colvec>(
+    (*cost_gradient)(data.rows(segment_start, segment_end), theta)
+  );
+}
+
 colvec Fastcpd::get_gradient_lm(
   const unsigned int segment_start,
   const unsigned int segment_end,
@@ -109,8 +133,7 @@ colvec Fastcpd::get_gradient_ma(
     psi_coefficient.row(segment_length - 1).t() *
     variance_term(segment_length - 1) / theta(q);
   gradient(q) = 1.0 / 2.0 / theta(q) -
-    std::pow(variance_term(segment_length - 1), 2) / 2.0 /
-    std::pow(theta(q), 2);
+    pow(variance_term(segment_length - 1), 2) / 2.0 / pow(theta(q), 2);
   return gradient;
 }
 
@@ -218,9 +241,8 @@ mat Fastcpd::get_hessian_arma(
   hessian.submat(sum(order), order(0), sum(order), sum(order) - 1) =
     hessian.submat(order(0), sum(order), sum(order) - 1, sum(order)).t();
   hessian(sum(order), sum(order)) =
-    std::pow(variance_term(segment_length - 1), 2) /
-    std::pow(theta(sum(order)), 3) -
-    1.0 / 2.0 / std::pow(theta(sum(order)), 2);
+    pow(variance_term(segment_length - 1), 2) / pow(theta(sum(order)), 3) -
+    1.0 / 2.0 / pow(theta(sum(order)), 2);
   return hessian;
 }
 
@@ -237,6 +259,16 @@ mat Fastcpd::get_hessian_binomial(
   return (x.t() * x) * ((1 - prob) * prob);
 }
 
+mat Fastcpd::get_hessian_custom(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  const colvec& theta
+) {
+  return as<mat>(
+    (*cost_hessian)(data.rows(segment_start, segment_end), theta)
+  );
+}
+
 mat Fastcpd::get_hessian_lm(
   const unsigned int segment_start,
   const unsigned int segment_end,
@@ -303,9 +335,8 @@ mat Fastcpd::get_hessian_ma(
     variance_term(segment_length - 1) / theta(q) / theta(q);
   hessian.submat(q, 0, q, q - 1) = hessian.submat(0, q, q - 1, q).t();
   hessian(q, q) =
-    std::pow(variance_term(segment_length - 1), 2) /
-    std::pow(theta(q), 3) -
-    1.0 / 2.0 / std::pow(theta(q), 2);
+    pow(variance_term(segment_length - 1), 2) / pow(theta(q), 3) -
+    1.0 / 2.0 / pow(theta(q), 2);
   return hessian;
 }
 
@@ -323,9 +354,12 @@ mat Fastcpd::get_hessian_poisson(
   return (x.t() * x) * std::min(as_scalar(prob), 1e10);
 }
 
-CostResult Fastcpd::get_nll_arma(
+CostResult Fastcpd::get_nll_pelt_arma(
   const unsigned int segment_start,
-  const unsigned int segment_end
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
   const mat data_segment = data.rows(segment_start, segment_end);
   Environment stats = Environment::namespace_env("stats");
@@ -339,16 +373,36 @@ CostResult Fastcpd::get_nll_arma(
   par.rows(0, sum(order) - 1) = as<colvec>(out["coef"]);
   par(sum(order)) = as<double>(out["sigma2"]);
 
-  return {{par}, {as<vec>(out["residuals"])}, -as<double>(out["loglik"])};
+  return {{par}, {as<colvec>(out["residuals"])}, -as<double>(out["loglik"])};
 }
 
-CostResult Fastcpd::get_nll_glm(
+CostResult Fastcpd::get_nll_pelt_custom(
   const unsigned int segment_start,
   const unsigned int segment_end,
-  Nullable<colvec> start
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
+) {
+  if (cost_gradient || cost_hessian) {
+    return get_optimized_cost(segment_start, segment_end);
+  } else {
+    return {
+      {colvec()},
+      {colvec()},
+      as<double>((*cost)(data.rows(segment_start, segment_end)))
+    };
+  }
+}
+
+CostResult Fastcpd::get_nll_pelt_glm(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
   const mat data_segment = data.rows(segment_start, segment_end);
-  vec y = data_segment.col(0);
+  colvec y = data_segment.col(0);
   Environment fastglm = Environment::namespace_env("fastglm");
   Function fastglm_ = fastglm["fastglm"];
   List out;
@@ -360,97 +414,105 @@ CostResult Fastcpd::get_nll_glm(
     mat x = data_segment.cols(1, data_segment.n_cols - 1);
     out = fastglm_(x, y, family, Named("start") = start_);
   }
-  vec par = as<vec>(out["coefficients"]);
-  vec residuals = as<vec>(out["residuals"]);
+  colvec par = as<colvec>(out["coefficients"]);
+  colvec residuals = as<colvec>(out["residuals"]);
   double value = out["deviance"];
   return {{par}, {residuals}, value / 2};
 }
 
-CostResult Fastcpd::get_nll_lasso_cv(
-  const unsigned int segment_start,
-  const unsigned int segment_end
-) {
-  const mat data_segment = data.rows(segment_start, segment_end);
-  Environment glmnet = Environment::namespace_env("glmnet"),
-               stats = Environment::namespace_env("stats");
-  Function cv_glmnet = glmnet["cv.glmnet"],
-      predict_glmnet = glmnet["predict.glmnet"],
-            deviance = stats["deviance"];
-  List out = cv_glmnet(
-    data_segment.cols(1, data_segment.n_cols - 1),
-    data_segment.col(0),
-    Named("family") = "gaussian"
-  );
-  colvec index_vec = as<colvec>(out["index"]),
-            values = as<colvec>(deviance(out["glmnet.fit"]));
-  S4 out_coef = predict_glmnet(
-    out["glmnet.fit"],
-    Named("s") = out["lambda.1se"],
-    Named("type") = "coefficients",
-    Named("exact") = false
-  );
-  vec glmnet_i = as<vec>(out_coef.slot("i"));
-  vec glmnet_x = as<vec>(out_coef.slot("x"));
-  vec par = zeros(data_segment.n_cols - 1);
-  for (unsigned int i = 1; i < glmnet_i.n_elem; i++) {
-    par(glmnet_i(i) - 1) = glmnet_x(i);
-  }
-  return {{par}, {mat()}, values(index_vec(1) - 1)};
-}
-
-CostResult Fastcpd::get_nll_lasso_wo_cv(
+CostResult Fastcpd::get_nll_pelt_lasso(
   const unsigned int segment_start,
   const unsigned int segment_end,
-  const double lambda
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
-  const mat data_segment = data.rows(segment_start, segment_end);
-  Environment stats = Environment::namespace_env("stats"),
-             glmnet = Environment::namespace_env("glmnet");
-  Function deviance = stats["deviance"], glmnet_ = glmnet["glmnet"],
-     predict_glmnet = glmnet["predict.glmnet"];
-  List out = glmnet_(
-    data_segment.cols(1, data_segment.n_cols - 1), data_segment.col(0),
-    Named("family") = "gaussian", Named("lambda") = lambda
-  );
-  S4 out_par = out["beta"];
-  vec par_i = as<vec>(out_par.slot("i"));
-  vec par_x = as<vec>(out_par.slot("x"));
-  vec par = zeros(data_segment.n_cols - 1);
-  for (unsigned int i = 0; i < par_i.n_elem; i++) {
-    par(par_i(i)) = par_x(i);
+  if (cv) {
+    const mat data_segment = data.rows(segment_start, segment_end);
+    Environment glmnet = Environment::namespace_env("glmnet"),
+                stats = Environment::namespace_env("stats");
+    Function cv_glmnet = glmnet["cv.glmnet"],
+        predict_glmnet = glmnet["predict.glmnet"],
+              deviance = stats["deviance"];
+    List out = cv_glmnet(
+      data_segment.cols(1, data_segment.n_cols - 1),
+      data_segment.col(0),
+      Named("family") = "gaussian"
+    );
+    colvec index_vec = as<colvec>(out["index"]),
+              values = as<colvec>(deviance(out["glmnet.fit"]));
+    S4 out_coef = predict_glmnet(
+      out["glmnet.fit"],
+      Named("s") = out["lambda.1se"],
+      Named("type") = "coefficients",
+      Named("exact") = false
+    );
+    colvec glmnet_i = as<colvec>(out_coef.slot("i"));
+    colvec glmnet_x = as<colvec>(out_coef.slot("x"));
+    colvec par = zeros(data_segment.n_cols - 1);
+    for (unsigned int i = 1; i < glmnet_i.n_elem; i++) {
+      par(glmnet_i(i) - 1) = glmnet_x(i);
+    }
+    return {{par}, {mat()}, values(index_vec(1) - 1)};
+  } else {
+    const mat data_segment = data.rows(segment_start, segment_end);
+    Environment stats = Environment::namespace_env("stats"),
+              glmnet = Environment::namespace_env("glmnet");
+    Function deviance = stats["deviance"], glmnet_ = glmnet["glmnet"],
+      predict_glmnet = glmnet["predict.glmnet"];
+    List out = glmnet_(
+      data_segment.cols(1, data_segment.n_cols - 1), data_segment.col(0),
+      Named("family") = "gaussian", Named("lambda") = lambda
+    );
+    S4 out_par = out["beta"];
+    colvec par_i = as<colvec>(out_par.slot("i"));
+    colvec par_x = as<colvec>(out_par.slot("x"));
+    colvec par = zeros(data_segment.n_cols - 1);
+    for (unsigned int i = 0; i < par_i.n_elem; i++) {
+      par(par_i(i)) = par_x(i);
+    }
+    double value = as<double>(deviance(out));
+    colvec fitted_values = as<colvec>(
+      predict_glmnet(
+        out, data_segment.cols(1, data_segment.n_cols - 1), Named("s") = lambda
+      )
+    );
+    colvec residuals = data_segment.col(0) - fitted_values;
+    return {{par}, {residuals}, value / 2};
   }
-  double value = as<double>(deviance(out));
-  vec fitted_values = as<vec>(
-    predict_glmnet(
-      out, data_segment.cols(1, data_segment.n_cols - 1), Named("s") = lambda
-    )
-  );
-  vec residuals = data_segment.col(0) - fitted_values;
-  return {{par}, {residuals}, value / 2};
 }
 
-CostResult Fastcpd::get_nll_mean(
+CostResult Fastcpd::get_nll_pelt_mean(
   const unsigned int segment_start,
-  const unsigned int segment_end
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
-  const rowvec data_diff =
-    zero_data.row(segment_end + 1) - zero_data.row(segment_start);
+  double two_norm = 0;
+  for (unsigned int i = 0; i < p; i++) {
+    two_norm +=
+      (zero_data_c[segment_end + 1][i] - zero_data_c[segment_start][i]) *
+      (zero_data_c[segment_end + 1][i] - zero_data_c[segment_start][i]);
+  }
   const unsigned int segment_length = segment_end - segment_start + 1;
   return {
-    {zeros<colvec>(p)},  // # nocov
+    {zeros<colvec>(p)},
     {zeros<mat>(segment_length, p)},
-    std::log(2.0 * M_PI) * zero_data.n_cols + log_det_sympd(variance_estimate) *
+    std::log(2.0 * M_PI) * data_n_cols + log_det_sympd(variance_estimate) *
       (segment_length) / 2.0 + (
-      data_diff(p) - dot(
-        data_diff.subvec(0, p - 1), data_diff.subvec(0, p - 1)
-      ) / segment_length
+      (zero_data_c[segment_end + 1][p] - zero_data_c[segment_start][p]) -
+      two_norm / segment_length
     ) / 2.0
   };
 }
 
-CostResult Fastcpd::get_nll_meanvariance(
+CostResult Fastcpd::get_nll_pelt_meanvariance(
   const unsigned int segment_start,
-  const unsigned int segment_end
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
   rowvec data_diff =
     zero_data.row(segment_end + 1) - zero_data.row(segment_start);
@@ -486,13 +548,16 @@ CostResult Fastcpd::get_nll_meanvariance(
   return {
     {zeros<colvec>(p)},
     {mat()},
-    (d * std::log(2.0 * M_PI) + d + log(det_value)) * (segment_length) / 2.0
+    (d * std::log(2.0 * M_PI) + d + log(det_value)) * segment_length / 2.0
   };
 }
 
-CostResult Fastcpd::get_nll_mgaussian(
+CostResult Fastcpd::get_nll_pelt_mgaussian(
   const unsigned int segment_start,
-  const unsigned int segment_end
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
   const mat data_segment = data.rows(segment_start, segment_end);
   mat x = data_segment.cols(p_response, data_segment.n_cols - 1);
@@ -516,9 +581,12 @@ CostResult Fastcpd::get_nll_mgaussian(
   return {{par}, {residuals}, value / 2};
 }
 
-CostResult Fastcpd::get_nll_variance(
+CostResult Fastcpd::get_nll_pelt_variance(
   const unsigned int segment_start,
-  const unsigned int segment_end
+  const unsigned int segment_end,
+  const double lambda,
+  const bool cv,
+  const Nullable<colvec>& start
 ) {
   const unsigned int segment_length = segment_end - segment_start + 1;
 
@@ -551,4 +619,113 @@ CostResult Fastcpd::get_nll_variance(
   };
 }
 
+double Fastcpd::get_nll_sen_arma(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  mat data_segment = data.rows(segment_start, segment_end);
+  colvec reversed_theta = reverse(theta);
+  if (data_segment.n_rows < max(order) + 1) {
+    return 0;
+  }
+  colvec variance_term = zeros(data_segment.n_rows);
+  for (unsigned int i = max(order); i < data_segment.n_rows; i++) {
+    variance_term(i) = data_segment(i, 0) - dot(
+        reversed_theta.rows(order(1) + 1, sum(order)),
+        data_segment.rows(i - order(0), i - 1)
+      ) - dot(
+        reversed_theta.rows(1, order(1)),
+        variance_term.rows(i - order(1), i - 1)
+      );
+  }
+  return (std::log(2.0 * M_PI) +
+    std::log(theta(sum(order)))) * (data_segment.n_rows - 2) / 2.0 +
+    dot(variance_term, variance_term) / 2.0 / theta(sum(order));
+}
+
+double Fastcpd::get_nll_sen_binomial(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  mat data_segment = data.rows(segment_start, segment_end);
+  colvec y = data_segment.col(0);
+  // Calculate negative log likelihood in binomial family
+  mat x = data_segment.cols(1, data_segment.n_cols - 1);
+  colvec u = x * theta;
+  return accu(-y % u + arma::log(1 + exp(u)));
+}
+
+double Fastcpd::get_nll_sen_custom(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  return as<double>(
+    (*cost)(data.rows(segment_start, segment_end), theta)
+  );
+}
+
+double Fastcpd::get_nll_sen_lm(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  mat data_segment = data.rows(segment_start, segment_end);
+  colvec y = data_segment.col(0);
+  // Calculate negative log likelihood in gaussian family
+  double penalty = lambda * accu(abs(theta));
+  mat x = data_segment.cols(1, data_segment.n_cols - 1);
+  return accu(square(y - x * theta)) / 2 + penalty;
+}
+
+double Fastcpd::get_nll_sen_ma(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  mat data_segment = data.rows(segment_start, segment_end);
+  const unsigned int q = order(1);
+  colvec reversed_theta = reverse(theta);
+  if (data_segment.n_rows < q + 1) {
+    return 0;
+  }
+  colvec variance_term = zeros(data_segment.n_rows);
+  for (unsigned int i = q; i < data_segment.n_rows; i++) {
+    variance_term(i) = data_segment(i, 0) - dot(
+        reversed_theta.rows(1, q), variance_term.rows(i - q, i - 1)
+      );
+  }
+  return (std::log(2.0 * M_PI) +
+    std::log(theta(q))) * (data_segment.n_rows - 2) / 2.0 +
+    dot(variance_term, variance_term) / 2.0 / theta(q);
+}
+
+double Fastcpd::get_nll_sen_poisson(
+  const unsigned int segment_start,
+  const unsigned int segment_end,
+  colvec theta,
+  double lambda
+) {
+  mat data_segment = data.rows(segment_start, segment_end);
+  colvec y = data_segment.col(0);
+  mat x = data_segment.cols(1, data_segment.n_cols - 1);
+  colvec u = x * theta;
+  colvec y_factorial(y.n_elem);
+  for (unsigned int i = 0; i < y.n_elem; i++) {
+    double log_factorial = 0;
+    for (int j = 1; j <= y(i); ++j) {
+      log_factorial += std::log(j);
+    }
+    y_factorial(i) = log_factorial;
+  }
+  return accu(-y % u + exp(u) + y_factorial);
+}
+
 }  // namespace fastcpd::classes
