@@ -1,7 +1,9 @@
 testthat::test_that(
   "examples/fastcpd_lasso.txt", {
+    testthat::skip_if_not_installed("dplyr")
     testthat::skip_if_not_installed("ggplot2")
     testthat::skip_if_not_installed("mvtnorm")
+    testthat::skip_if_not_installed("reshape2")
 
     examples_lasso <- readLines("examples/fastcpd_lasso.txt")
     source(textConnection(paste(
